import os
import argparse
import asyncio
from spydy.engine import Engine
from spydy.urls import DummyUrls
from spydy.request import AsyncHttpRequest
from spydy.logs import StatsReportLog
from spydy.utils import check_configs


def main():
    arg_parser = argparse.ArgumentParser(
        prog="py-wrk",
        description="Small http benchmarking tool written in python",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    # prepare args
    arg_parser.add_argument("url", type=str, help="Url")

    arg_parser.add_argument(
        "-m", "--method", type=str, help="Http method", default="Get"
    )

    arg_parser.add_argument("-p", "--params", type=str, help="Request Parameters")

    arg_parser.add_argument("-d", "--data", type=str, help="Request body")

    arg_parser.add_argument("-H", "--headers", type=str, help="Request header")

    arg_parser.add_argument("--cookies", type=str, help="Cookies")

    arg_parser.add_argument("--user", type=str, help="User for Authentication")

    arg_parser.add_argument("--password", type=str, help="Passworf for Authentication")

    arg_parser.add_argument(
        "-c",
        "--connections",
        type=int,
        help="Number of Connetions at same time",
        default=10,
    )

    arg_parser.add_argument(
        "-n", "--numbers", type=int, help="Totoal numbers to call", default=100
    )
    # Parse args
    args = arg_parser.parse_args()

    url = args.url
    method = args.method
    connections = args.connections
    numbers = args.numbers
    user = args.user
    password = args.password
    if user and password:
        auth = (user, password)
    else:
        auth = None

    # Parse dict-like string to python dict
    params = args.params
    try:
        if params:
            params = eval(params)
    except:
        raise ValueError("Can not parse params")
    data = args.data
    try:
        if data:
            data = eval(data)
    except:
        raise ValueError("Can not parse data")
    headers = args.headers
    try:
        if headers:
            headers = eval(headers)
    except:
        raise ValueError("Can not parse headers")
    cookies = args.cookies
    try:
        if cookies:
            cookies = eval(cookies)
    except:
        raise ValueError("Can not parse cookies")

    spydy_config = {
        "Globals": {
            "run_mode": "async_forever",
            "nworkers": connections,
            "verbose": "False",
        },
        "PipeLine": [
            DummyUrls(url=url, repeat=numbers),
            AsyncHttpRequest(
                method=method,
                params=params,
                data=data,
                headers=headers,
                cookies=cookies,
                auth=auth
            ),
            StatsReportLog(every=int(numbers / 10)),
        ],
    }
    check_configs(spydy_config)
    spider = Engine.from_dict(spydy_config)
    spider.run()



from setuptools import setup, find_packages
from codecs import open
from os import path


__version__ = "0.1.0"

here = path.abspath(path.dirname(__file__))

long_description_for_pypi = """

"""

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# get the dependencies and installs
with open(path.join(here, "requirements.txt"), encoding="utf-8") as f:
    all_reqs = f.read().split("\n")

install_requires = [x.strip() for x in all_reqs if "git+" not in x]
dependency_links = [
    x.strip().replace("git+", "") for x in all_reqs if x.startswith("git+")
]

setup(
    name="pywrk",
    version=__version__,
    description="Small http benchmarking tool written in python",
    long_description=long_description_for_pypi,
    long_description_content_type="text/markdown",
    url="https://github.com/superjcd/pywrk",
    download_url="https://github.com/superjcd/py-wrk/tarball/" + __version__,
    license="BSD",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
    ],
    keywords="",
    entry_points={
        "console_scripts": [
            "pywrk=wrk.main:main",
        ],
    },
    packages=find_packages(exclude=["docs", "tests*"]),
    include_package_data=True,
    author="Jiang Chaodi",
    install_requires=install_requires,
    dependency_links=dependency_links,
    author_email="929760274@qq.com",
)
